% Before running this, please, run c# version to generate tests.
s = dir('Tests\*.csv');
testCount = size(s,1);
% metrics to test, this must be synchronized with c# version.
metrics = {'cityblock'; 'euclidean'; 'seuclidean'; 'chebychev'; 'cosine'; 'hamming'; 'correlation'};
metricsCount = size(metrics,1);

fout = fopen('matlab-results.csv','w');

for j=1:metricsCount 
    metric = char(metrics(j));
    fprintf('\n');
    fprintf('_____________________________________________________________________\n');
    fprintf('| %65s \n', metric);
    fprintf('_____________________________________________________________________\n');
    fprintf('| Objects  |  Factors | Repeats |   Total   |  Average  |  Single   |\n');
	fprintf('|  count   |   count  |  count  |   Time    |  MFLOPS   | Calc Time |\n');
    fprintf('_____________________________________________________________________\n');
    fprintf(fout, '%s\n',metric);
    fprintf(fout,'Objects,Factors,Repeats,Matlab Time,MatLab MFLOPS,Single calc time\n');
    for i=1:testCount
        test = s(i);
        fileName = test.name; 
        temp = sscanf(fileName, '%d.%d.%d.%d');
        o = temp(2);
        f = temp(3);
        r = temp(4);
        op = o * o * f * r;
        X = dlmread(strcat('Tests\', fileName));
        tic;
        for l=1:r
            A = linkage(X, 'single', metrics(j));
            clear A;
        end;
        time = toc;
        p = op / time / 1e+6;
        it = time / r;
        fprintf('| %8d | %8d | %7d |    %6.3f | %9d |    %6.3f |\n', o, f, r, time, ceil(p), it);
        fprintf(fout,'%d,%d,%d,%6.3f,%d,%6.3f\n',o, f, r, time, ceil(p), it);
        fprintf('_____________________________________________________________________\n');
    end;
end;
fclose(fout);

ffinmath = fopen('finmath-results.csv','r');
fmatlab = fopen('matlab-results.csv','r');

s = dir('Tests\*.csv');
testCount = size(s,1);

objects = zeros(testCount, 1);
factors = zeros(testCount, 1);
Ftime = zeros(testCount, metricsCount );
Fmflops = zeros(testCount, metricsCount );
Fperrun = zeros(testCount, metricsCount );
names = cell(metricsCount, 1);
Mtime = zeros(testCount, metricsCount );
Mmflops = zeros(testCount, metricsCount );
Mperrun = zeros(testCount, metricsCount );

for j=1:metricsCount 
    names(j) = cellstr(fgets(ffinmath));
    fgets(ffinmath);
    fgets(fmatlab);
    fgets(fmatlab);
    for i=1:testCount
        s = fgets(ffinmath);
        t = sscanf(s, '%d,%d,%d,%f,%d,%f\n');
        Ftime(i,j) = t(4);
        Fmflops(i,j) = t(5);
        Fperrun(i,j) = t(6);
        
        s = fgets(fmatlab);
        t = sscanf(s, '%d,%d,%d,%f,%d,%f\n');
        Mtime(i,j) = t(4);
        Mmflops(i,j) = t(5);
        Mperrun(i,j) = t(6);
        objects(i) = t(1);
        factors(i) = t(2);
    end;
end;

fclose(ffinmath);
fclose(fmatlab);

cols = unique(objects);
colsCount = length(cols);

figure(1);
for j=1:metricsCount 
    for i=1:colsCount
        indices = find(objects == cols(i));
        fff = Fperrun(indices,j);
        mmm = Mperrun(indices,j);
        subplot(metricsCount, colsCount, i + (j-1)*colsCount), pf = plot(factors(indices), fff);
        set(pf,'Color','blue','LineWidth',1)
        hold all;
        subplot(metricsCount, colsCount, i + (j-1)*colsCount), pm = plot(factors(indices), mmm);
        set(pm,'Color','red','LineWidth',1)
        title(strcat(names(j),'  ',int2str(cols(i))));
        xlabel('Factors count');
        ylabel('Time in sec.');
        if (i == 1 && j == 1)
            leg = legend('FinMath','MatLab');
            set(leg,'Location','NorthWest');
        end;
    end;
end;
